/**
 * Copyright (C) 2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

// ------ I N C L U D E   F I L E S -------------------------------------------
// Local - Include Files
#include "ReportPcieInfo.h"
#include "MAUtilities.h"

// 3rd Party Library - Include Files
#include <boost/property_tree/json_parser.hpp>

void ReportPcieInfo::getPropertyTree(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& _pt) const {
  boost::property_tree::ptree pt;

  XBUtilities::get_pcie_info(dev, pt);

  // There can only be 1 root node
  _pt.add_child("pcie_info", pt);
}

void ReportPcieInfo::writeReport(const boost::property_tree::ptree& _pt, std::ostream& _output) const {
  _output << "Pcie Info:\n";
  auto& pt_pcie = _pt.get_child("pcie_info.pcie_info");
  if (pt_pcie.empty()) {
    _output << "  Information unavailable" << std::endl;
    return;
  }
  if (_pt.get<std::string>("pcie_info.status") != "ok") {
    _output << "ERROR " << _pt.get<std::string>("pcie_info.status") << std::endl;
    return;
  }

  _output << boost::format("  %-22s : %s\n") % "Vendor" % pt_pcie.get<std::string>("vendor_id");
  _output << boost::format("  %-22s : %s\n") % "Device" % pt_pcie.get<std::string>("device_id");
  _output << boost::format("  %-22s : %s, Width %s\n") % "PCIe" % pt_pcie.get<std::string>("speed", "?") % pt_pcie.get<std::string>("width", "?");
  auto rx_bw = pt_pcie.get_optional<std::string>("rx_bw_MBps");
  if (rx_bw) {
    _output << boost::format("  %-22s : %s MBps\n") % "RX Bandwidth" % rx_bw.get();
    _output << boost::format("  %-22s : %s MBps\n") % "TX Bandwidth" % pt_pcie.get<std::string>("tx_bw_MBps", "?");
  }
  _output << std::endl;
}
