/**
 * Copyright (C) 2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

// Sub Commands
#include "SubCmdProgram.h"
#include "SubCmdReset.h"
#include "SubCmdExamine.h"
#include "SubCmdCreateVF.h"
#include "SubCmdFlashVerify.h"

// Supporting tools
#include "../common/MAMain.h"
#include "../common/SubCmd.h"

// System include files
#include <boost/filesystem.hpp>
#include <string>
#include <iostream>
#include <exception>

// Program entry
int main(int argc, char** argv) {
  // -- Build the supported subcommands
  SubCmdsCollection subCommands;

  {
    // Syntax: SubCmdClass( IsHidden, IsDepricated, IsPreliminary)
    subCommands.emplace_back(std::make_shared<SubCmdProgram>(false, false, false));
    subCommands.emplace_back(std::make_shared<SubCmdFlashVerify>(false, false, false));
    subCommands.emplace_back(std::make_shared<SubCmdReset>(false, false, false));
    subCommands.emplace_back(std::make_shared<SubCmdCreateVF>(false, false, false));
    subCommands.emplace_back(std::make_shared<SubCmdExamine>(false, false, false));
  }

  const std::string executable = "mamgmt";

  for (auto& subCommand : subCommands)
    subCommand->setExecutableName(executable);

  // -- Program Description
  const std::string description =
      "The MA35 Board Management (mamgmt) is a standalone command line utility that"
      " is included with the MA35 installation package.";

  // -- Ready to execute the code
  try {
    main_(argc, argv, executable, description, subCommands);
    return 0;
  } catch (const std::exception& e) {
    std::cerr << "  ERROR: " << e.what() << std::endl;
  } catch (...) {
    std::cerr << "ERROR Occurred: " << std::endl;
  }

  return 1;
}
